package org.sinev.igor.applications;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashSet;

import org.sinev.igor.collections.ThreadedList;
import org.sinev.igor.ps.PSParseException;
import org.sinev.igor.ps.elementtype.ElementType;
import org.sinev.igor.ps.model.*;
import org.sinev.igor.ps.model.impl.ElementFactory;
import org.sinev.igor.ps.reader.PSReader;

/**
 * Utility class.
 * Doubles dimensions of all displayed elements in the file.
 * Only operators included in default operator description 
 * {@link org.sinev.igor.ps.model.DocumentFactory#DEFAULT_NAMING_MAP}
 * list are supported.
 * 
 * @author Igor Sinev 
 */
public class SizesDoubler {

	public static void main(String[] args) {
		if (args.length != 2) {
			System.err.println("usage:\n  java " + SizesDoubler.class.getName() + " <input file name> <output file name>");
			return;
		}

		IDocument psDocument;
		try {			
			psDocument = DocumentFactory.getDocumentFromReader(new PSReader(new FileReader(args[0])));			
			doubleSizes(psDocument);
			psDocument.write(new FileWriter(new File(args[1])));
			
		} catch (FileNotFoundException e) {			
			e.printStackTrace();
		} catch (PSParseException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}		
	}

	/**
	 * Doubles dimensions of all displayed elements in specified document.
	 * 
	 * @param d target document.
	 * 
	 * @throws PSParseException
	 */
	public static void doubleSizes(IDocument d) throws PSParseException {
		HashSet<IFunctionalElement> toBeDoubled = new HashSet<IFunctionalElement>();
		for (IElement e : d.getElementsList()) {			
			if (e instanceof IPSEOperator) {
				IPSEOperator o = (IPSEOperator) e;
				if (o.getDescription().getGroup().indexOf("Path construction") == -1)
					continue;
				for (IStackObject so : o.getArguments()) {
					if (!so.getType().matches(ElementType.NUMBER))
						continue;
					if (so instanceof ICalculatedStackObject){						
						toBeDoubled.add(((ICalculatedStackObject) so).getFunctionalElement());
					} else if (so.getType() == ElementType.REAL) {
						IPSEReal sor = (IPSEReal) so;
						sor.setValue(2 * sor.getValue());
					} else if (so.getType() == ElementType.REAL) {
						IPSEInteger soi = (IPSEInteger) so;
						soi.setValue(2 * soi.getValue());
					} else {						
						throw new IllegalStateException(
								"Illegal argument for path construction operator " +
								"(type is " + so.getClass().getName() + ")."
							);
					}
				}
			}
		}
		
		for (ThreadedList.ThreadedListIterator ei = d.getBody().threadedListIterator(); ei.hasNext(); ) {
			IElement e = (IElement) ei.next();			
			if (toBeDoubled.contains(e)) {
				ei.add(ElementFactory.createOperatorElement("mul", d));
				ei.add(ElementFactory.createWhitespaceElement(" ", d));
				ei.add(ElementFactory.createIntegerElement("2", d));
				ei.add(ElementFactory.createWhitespaceElement(" ", d));				
			}
		}
		
		d.buildOperatorLinks();
	}

}
