package org.sinev.igor.applications;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashSet;

import org.sinev.igor.collections.ThreadedList;
import org.sinev.igor.ps.PSParseException;
import org.sinev.igor.ps.model.DocumentFactory;
import org.sinev.igor.ps.model.IDocument;
import org.sinev.igor.ps.model.IElement;
import org.sinev.igor.ps.model.IPSEOperator;
import org.sinev.igor.ps.model.impl.ElementFactory;
import org.sinev.igor.ps.reader.PSReader;

/**
 * Utility class.
 * Swaps coorinates of all displayed elements in the file.
 * Only operators included in default operator description 
 * {@link org.sinev.igor.ps.model.DocumentFactory#DEFAULT_NAMING_MAP} 
 * list are supported.
 * 
 * @author Igor Sinev 
 */
public class XYSwapper {

	public static void main(String[] args) {
		if (args.length != 2) {
			System.err.println("usage:\n  java " + XYSwapper.class.getName() + " <input file name> <output file name>");
			return;
		}

		IDocument psDocument;
		try {			
			psDocument = DocumentFactory.getDocumentFromReader(new PSReader(new FileReader(args[0])));			
			swapXY(psDocument);
			psDocument.write(new FileWriter(new File(args[1])));
			
		} catch (FileNotFoundException e) {			
			e.printStackTrace();
		} catch (PSParseException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}		
	}
	
	/**
	 * Swaps coorinates of all displayed elements in specified document.
	 * 
	 * @param d target document.
	 * 
	 * @throws PSParseException
	 */
	public static void swapXY(IDocument d) throws PSParseException {
		HashSet<IPSEOperator> toBeChanged = new HashSet<IPSEOperator>();
		for (IElement e : d.getElementsList()) {			
			if (e instanceof IPSEOperator) {
				IPSEOperator o = (IPSEOperator) e;
				if (o.getDescription().getGroup().indexOf("Path construction") == -1)
					continue;							
				toBeChanged.add(o);									
			}
		}
		
		for (ThreadedList.ThreadedListIterator ei = d.getBody().threadedListIterator(); ei.hasNext(); ) {
			IElement e = (IElement) ei.next();			
			if (toBeChanged.contains(e)) {
				ei.add(ElementFactory.createOperatorElement(e.getRepresentation(), d));				
				ei.add(ElementFactory.createWhitespaceElement(" ", d));							
				ei.replace(ElementFactory.createOperatorElement("exch", d));
			}
		}
		
		d.buildOperatorLinks();
	}

}
