package org.sinev.igor.collections;

import java.util.AbstractList;
import java.util.Iterator;

import org.sinev.igor.ps.model.IElement;
import org.sinev.igor.ps.model.impl.Element;

/** 
 * Internal-use collection that contains linked list of elements 
 * with possibility to acquire an iterator by its element.
 *  
 * @author Igor Sinev
 *
 */
public class ThreadedList extends AbstractList<IElement> {
	AbstractList<IElement> a;
	/**
	 * Entry that wraps one element of the list.
	 * 
	 * @author Igor Sinev	 
	 */
	public class ListEntry {
		ListEntry myPrev;
		ListEntry myNext;
		IElement myValue;
		public ListEntry(IElement value) {
			this.myValue = value;
		}
		@Override
		public String toString() {
			return myPrev.myValue.toString() + " -> " + myValue.toString() + " <- " + myNext.myValue.toString();
		}		
	}
	
	private ListEntry myHead;
	private ListEntry myTail;
	
	private int mySize = 0;
	
	/**
	 * Iterator for the list.
	 * 
	 * @author Igor Sinev
	 */
	public class ThreadedListIterator implements Iterator<IElement> {
		
		private ListEntry myLastElement = null;
		private ListEntry myCurrentElement;
		

		public ThreadedListIterator(ListEntry e) {
			myCurrentElement = e;
		}
		
		public ThreadedListIterator() {
			this(myHead);
		}		

		public boolean hasNext() {
			return (myCurrentElement != null);
		}

		public IElement next() {
			myLastElement = myCurrentElement;
			myCurrentElement = myCurrentElement.myNext;
			return myLastElement.myValue;			
		}

		public void remove() {
			mySize--;
			if (myLastElement == null)
				throw new IllegalStateException();
			deleteEntry(myLastElement);
		}
		
		public void add(IElement o) {
			addAfter(myLastElement, o);
		}

		public void replace(IElement o) {
			myLastElement.myValue = o;			
		}				
	}
	
	private void bind(ListEntry a, ListEntry b) {
		if (a != null)
			a.myNext = b;
		if (b != null)
			b.myPrev = a;
	}
	
	private void deleteEntry(ListEntry e) {
		ListEntry p = e.myPrev;
		ListEntry n = e.myNext;
		if (e == myHead)
			e = n;
		if (e == myTail)
			e = p;
		bind(p, n);
	}

	public Iterator<IElement> iterator() {
		return threadedListIterator();
	}
	
	/**
	 * Gets native iterator for this list.
	 * 
	 * @return {@link ThreadedListIterator} for this list.
	 */
	public ThreadedListIterator threadedListIterator() {
		return new ThreadedListIterator();
	}
	
	/**
	 * Gets native iterator for this list.
	 * 
	 * @return {@link ThreadedListIterator} for this list.
	 */
	public ThreadedListIterator threadedListIterator(ListEntry e) {
		return new ThreadedListIterator(e);
	}

	@Override
	public int size() {
		return mySize;
	}

	@Override
	public boolean add(IElement ieo) {
		Element eo = (Element) ieo;
		ListEntry e = new ListEntry(eo);
		eo.setListEntry(e);
		mySize++;
		if (mySize == 1) {
			myHead = e;
			myTail = e;
			return true;
		}
		myTail.myNext = e;
		e.myPrev = myTail;
		myTail = e;
		return true;
	}	
	
	private boolean addAfter(ListEntry ea, IElement b) {
		if (ea == myTail)
			return add(b);
		ListEntry eb = new ListEntry(b);
		mySize++;
		ListEntry e = ea.myNext;
		bind(ea, eb);
		bind(eb, e);
		return true;
	}

	@Override
	public IElement get(int index) {		
		if (index < 0 || index >= size())
			throw new IndexOutOfBoundsException();
		int i = 0;
		for (IElement e : this) {
			if (i++ == index)
				return e;
		}
		return null;
	}
}