package org.sinev.igor.ps.model.impl;

import java.io.IOException;
import java.io.Writer;
import java.util.HashMap;
import java.util.Map;

import org.sinev.igor.ps.model.IDocument;
import org.sinev.igor.ps.model.IElement;
import org.sinev.igor.ps.operatordescription.OperatorNamingMap;

/**
 * PS document representation.
 * 
 * @author Igor 
 */
public class Document extends ElementStructure implements IDocument{
	
	/**
	 * Stores information about seen procedures.
	 */
	private Map<String, PSEProcedureElement> myProcedureNamingMap = new HashMap<String, PSEProcedureElement>();
	
	private OperatorNamingMap myOperatorNamingMap;
	
	/**
	 * Adds procedure description to the map.
	 * 
	 * @param n procedure name
	 * @param p procedure description
	 */
	public void addProcedure(String n, PSEProcedureElement p) {
		myProcedureNamingMap.put(n, p);	
	}
	
	/**
	 * Gets information about procedure.
	 * 
	 * @param n procedure name
	 * 
	 * @return corresponding procedure description
	 */
	public PSEProcedureElement getProcedure(String n) {
		return myProcedureNamingMap.get(n);
	}
	
	public Document() {
	}
	
	public void write(Writer w) throws IOException {
		for (IElement e : getElementsList()) {
			w.write(e.getRepresentation());
		}
		w.close();
	}

	public OperatorNamingMap getNamingMap() {
		return myOperatorNamingMap;
	}

	public void setNamingMap(OperatorNamingMap map) {
		myOperatorNamingMap = map;	
	}
		
}
