package org.sinev.igor.ps.model.impl;

import org.sinev.igor.ps.model.IElement;
import org.sinev.igor.ps.model.IStackObject;

/**
 * Utility class used to produce debug information.
 * 
 * @author Igor Sinev 
 */
public class DocumentDebugWriter {

	/**
	 * Writes degailed information in a list form about specified document to console
	 * 
	 * @param d document to write
	 */
	public static void listDebugOutput(Document d) {
		System.out.println(listDebugOutputString(d));
	}

	/**
	 * Gives detailed information in a list form about specified document
	 * 
	 * @param d document to write
	 */
	public static String listDebugOutputString(Document d) {
		StringBuffer t = new StringBuffer("Document: \n");		
		for (IElement e : d.getElementsList()) {
			((Element) e).listDebugOutput(t);
			t.append("\n");
		}
		return t.toString();
	}

	/**
	 * Prints structured debug representation of the document 
	 * to console.
	 * 
	 * @param d target document
	 */
	public static void structureDebugOutput(Document d) {
		System.out.println(structureDebugOutputString(d));
	}

	/**
	 * Creates structured debug representation of the document.
	 * 
	 * @param d target document
	 * 
	 * @return structured debug representation.
	 */
	public static String structureDebugOutputString(Document d) {
		StringBuffer b = new StringBuffer();
		for (IStackObject so : d.getOutgoingValues())
			callStructureDebugOutput(so, b);
		return b.toString();
	}
	/**
	 * Calls debug output for this object subtree.
	 * 
	 * @param target
	 */
	public static void callStructureDebugOutput(IStackObject o, StringBuffer target) {
		if (o instanceof CalculatedStackObject) {
			CalculatedStackObject c = (CalculatedStackObject) o;
			c.callStructureDebugOutput(target);			
		}
		else if (o instanceof Element) {
			Element e = (Element) o;
			e.structureDebugOutput(target);
		}
		else {
			throw new RuntimeException("Objects of type \'" + o.getClass().getName() + "\' have no implementtation to debug output");
		}
	}

}
