package org.sinev.igor.ps.model.impl;

import java.util.HashSet;

import org.sinev.igor.collections.ThreadedList;
import org.sinev.igor.collections.ThreadedList.ListEntry;
import org.sinev.igor.ps.PSParseException;
import org.sinev.igor.ps.model.IDocument;
import org.sinev.igor.ps.model.IElement;
import org.sinev.igor.ps.model.IStackObject;

/**
 * Generalized element of PS document.
 * 
 * @author Igor 
 */
public abstract class Element implements IElement, IStackObject {	
	/**
	 * Original string representation of this element.
	 */
	protected String myRepresentation;
	protected IDocument myDocument;
	
	private ThreadedList.ListEntry myListEntry;	
	
	
	/**
	 * Writes degailed information in a list form about this element to a buffer
	 * 
	 * @param target buffer to write to
	 */
	public void listDebugOutput(StringBuffer target) {		
		target.append(this.getClass().getSimpleName()).append(": ").append(myRepresentation);		
	}	
	
	// Used for debug output purposes
	protected static HashSet<Element> structureDebugOutputProcessedElements = new HashSet<Element>();
	protected static int structureDebugOutputRunningElements = 0;
	protected static int structureDebugOutputIndent = 0;

	
	/**
	 * Writes subtree of element (if any) to the buffer.
	 * 
	 * @param output buffer
	 */
	protected void structureDebugOutputWriteSubtree(StringBuffer target) {
		for (int i = 0; i < structureDebugOutputIndent; i++)
            target.append("  ");                     
        target.append(this).append(" (").append(getType()).append(")\n");		
	}
    /**
     * Produces structured debug output.
     * 
     * @param target buffer to write to
     */
    public void structureDebugOutput(StringBuffer target) {
    	if (structureDebugOutputProcessedElements.contains(this))
    		return;
    	structureDebugOutputProcessedElements.add(this);
        structureDebugOutputRunningElements++;        
        
        structureDebugOutputWriteSubtree(target);
        
        if (--structureDebugOutputRunningElements == 0)
            structureDebugOutputProcessedElements.clear();
    }
	
	protected Element(String representation, IDocument d) {
		myRepresentation = representation;
		myDocument = d;
	}

	public String getRepresentation() {
		return myRepresentation;
	}
	
	public String toString() {
		return myRepresentation;
	}
	
	public IElement getElement() {
		return this;
	}
	
	/**
	 * Creates argument links.  
	 * 
	 * @param s current interpretation stack. 
	 * @throws PSParseException 
	 */
	void putToStack(ElementStructure.StorageStack s) throws PSParseException {
		s.push(this);
	}

	/**
	 * Determines if this object should take part in argument 
	 * links structure.
	 *  
	 * @return true if this object is part of ordinarry
	 * structure.
	 */
	public abstract boolean isInterpretable();		
		
	public ListEntry getListEntry() {
		return myListEntry;
	}
	public void setListEntry(ListEntry listEntry) {
		this.myListEntry = listEntry;
	}
}
