package org.sinev.igor.ps.model.impl;

import org.sinev.igor.ps.elementtype.ElementType;
import org.sinev.igor.ps.model.IDocument;
import org.sinev.igor.ps.model.IPSECharacterString;

/**
 * Represents a character string element of some PS document.
 * Stores value as an unicode string.
 * 
 * @author Igor Sinev
 */
public class PSECharacterString extends Element implements IPSECharacterString {
	private String myValue;
	
	PSECharacterString(String representation, String v, IDocument d) {
		super(representation, d);
		myValue = v;
		//value = s.substring(1, s.length() - 1);
	}
	
	/**
	 * Creates a character string by value.
	 * 
	 * @param v value of a string
	 * @param d document for the element
	 * 
	 * @return created character string.
	 */
	static PSECharacterString createByValue(String v, Document d) {
		StringBuffer sb = new StringBuffer("(");
		for (int i = 0; i < v.length(); i++) {
			char c = v.charAt(i);
			switch (c) {
			case '\n':
				sb.append("\\n");
				break;
			case '\r':
				sb.append("\\r");
				break;
			case '\t':
				sb.append("\\r");
				break;
			case '\b':
				sb.append("\\b");
				break;
			case '\f':
				sb.append("\\f");
				break;
			case '\\':
				sb.append("\\\\");
				break;
			case '(':
			case ')':
				sb.append('\\').append(c);
				break;
			default:
				if (Character.isISOControl(c) || c > 127) {
					sb.append('\\').append(c / 64 + '0').append((c % 64) / 8).append(c % 8);
				} else {
					sb.append(c);
				}
			}
		}
		sb.append(')');
		return new PSECharacterString(sb.toString(), v, d);
	}
	
	public String getValue() {
		return myValue;
	}

	public ElementType getType() {
		return ElementType.CHARACTER_STRING;
	}
	
	@Override
	public boolean isInterpretable() {
		return true;
	}
}
