package org.sinev.igor.ps.model.impl;

import org.sinev.igor.ps.elementtype.ElementType;
import org.sinev.igor.ps.model.IDocument;
import org.sinev.igor.ps.model.IPSEHexadecimalString;

/**
 * Represents a hexadecimal string element of some PS document.
 * Stores value as an char array.
 * 
 * @author Igor Sinev
 */
public class PSEHexadecimalString extends Element implements IPSEHexadecimalString {
	private byte[] myValue;
	
	PSEHexadecimalString(String representation, byte[] v, IDocument d) {
		super(representation, d);
		myValue = v;		
	}
	
	private static final char writeHalfByte(byte b) {
		if (b < 10)
			return (char)('0' + b);
		else
			return (char)('a' + b - 10);
	}
		
	/**
	 * Creates a character string by value.
	 * 
	 * @param v value of a string.
	 * 
	 * @return created character string.
	 */
	static PSEHexadecimalString createByValue(byte[] v, Document d) {
		StringBuffer sb = new StringBuffer("<");
		for (int i = 0; i < v.length; i++) {
			byte b = v[i];
			b >>>= 4;
			sb.append(writeHalfByte(b));			
			b = v[i];
			b &= 15;
			sb.append(writeHalfByte(b));			
		}
		sb.append('>');
		return new PSEHexadecimalString(sb.toString(), v, d);
	}
	
	public ElementType getType() {
		return ElementType.HEXADECIMAL_STRING;
	}
	
	@Override
	public boolean isInterpretable() {
		return true;
	}
	
	/**
	 * Gets the value stored in the string
	 * 
	 * @return value of the string.
	 */
	public byte[] getValue() {
		return myValue;
	}
}
