package org.sinev.igor.ps.model.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.sinev.igor.ps.PSParseException;
import org.sinev.igor.ps.elementtype.ElementType;
import org.sinev.igor.ps.model.IDocument;
import org.sinev.igor.ps.model.IElement;
import org.sinev.igor.ps.model.IPSEProcedureElement;
import org.sinev.igor.ps.model.IStackObject;

/**
 * Procedure (executable array) element. Acts as a description 
 * for {@link ProcedureCall} elements.
 * 
 * @author Igor Sinev 
 */
public class PSEProcedureElement extends Element implements IPSEProcedureElement {		
	private ElementStructure myStructure;	

	protected PSEProcedureElement(String representation, IDocument d) {
		super(representation, d);
		throw new UnsupportedOperationException();
	}
	
	private ArrayList<ElementType> myReturns;

	public PSEProcedureElement(IElement[] contents, String representation, IDocument d) throws PSParseException {
		super(representation, d);
		myStructure = new ElementStructure();
		for (IElement e : contents)
			myStructure.addElement((Element) e);
		myStructure.buildOperatorLinks();
		
		List<IStackObject> ov = myStructure.getOutgoingValues();
		myReturns = new ArrayList<ElementType>(ov.size());		
		for (IStackObject iso : ov) {
			myReturns.add(iso.getType());
		}
	}
	
	public ElementType getType() {
		throw new IllegalStateException("Objects of type " + this.getClass().getName() + " can't take part in interpretation data flow as operator arguments");
	}
	
	/**
	 * Gets array of types this procedure takes as input values.
	 * 
	 * @return array of incoming types.
	 */
	public ElementType[] getIncomingTypes() {		
		return myStructure.myDebt.toArray(new ElementType[myStructure.myDebt.size()]);		
	}
	
	/**
	 * Gets array of types this procedure forms as output values.
	 * 
	 * @return list of outgoing types.
	 */
	public List<ElementType> getOutgoingTypes() {		
		return Collections.unmodifiableList(myReturns);
	}

	@Override
	public boolean isInterpretable() {		
		return false;
	}	
}
