package org.sinev.igor.ps.operatordescription;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.sinev.igor.ps.elementtype.ElementType;
import org.sinev.igor.ps.model.IElement;
import org.sinev.igor.ps.model.impl.Document;

/**
 * Contains specification of one operator.
 * 
 * @author Igor Sinev
 *
 */
public class OperatorDescription {
        
    private ArrayList<ElementType> myReturns;
    private ArrayList<ElementType> myArguments;
    private String myName;
    
    private String myGroup = "";
    private String myComment = "";
    
    private OperationPerformer myPerformer;
    /**
     * Sets an object that would perform calculation of this operator.
     * 
     * @param performer object that would perform calculation of this operator
     */
    public void setPerformer(OperationPerformer performer) {
    	this.myPerformer = performer;
    }        
    
    /**
     * Operator whose description wasn't found in the repository. 
     */
    public static final OperatorDescription undefinedOperator = new OperatorDescription("Undefined operator", new ElementType[0], new ElementType[0]);
    
    OperatorDescription(String n, ElementType[] a, ElementType[] r) {
        myName = n;
        myArguments = new ArrayList<ElementType>(a.length);
        for (ElementType t : a) 
            myArguments.add(t);
        myReturns = new ArrayList<ElementType>(r.length);
        for (ElementType t : r) 
            myReturns.add(t);
    }
    
    OperatorDescription(String n, ElementType[] a, ElementType[] r, String g, String d) {
        this(n, a, r);
        myGroup = g;
        myComment = d;  
    }
    
    /**
     * Gets list of types of the operator arguments.
     * 
     * @return argument types list.
     */
    public List<ElementType> getArguments() {
        return Collections.unmodifiableList(myArguments);       
    }
    
    /**
     * Gets the list of types of return values.
     * 
     * @return return values types list.
     */
    public List<ElementType> getReturns() {
        return Collections.unmodifiableList(myReturns);
    }
    
    /**
     * Gets name of the operator.
     * 
     * @return operator name
     */
    public String getName() {
    	return myName;
    }
    
    public String toString() {
        return myName;
    }
    
    
    /**
     * 
     * Allows to execute operator with this syntax.
     * 
     * @param args list that contains arguments of the operator.     
     * 
     * @return result of operator interpretation.
     */
    public List<IElement> calculate(ArrayList<IElement> args, Document d) {
        return myPerformer.calculate(args, d);
    }

    /**
     * Gets operator comment
     * 
     * @return operator comment
     */
    public String getComment() {
        return myComment;
    }
    
    /**
     * Gets information about operator group
     * 
     * @return operator group
     */
    public String getGroup() {
        return myGroup;
    }
    
}
