package org.sinev.igor.ps.tests;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StringWriter;

import junit.framework.TestCase;

import org.sinev.igor.applications.SizesDoubler;
import org.sinev.igor.applications.XYSwapper;
import org.sinev.igor.ps.PSParseException;
import org.sinev.igor.ps.model.DocumentFactory;
import org.sinev.igor.ps.model.IDocument;
import org.sinev.igor.ps.model.IReader;
import org.sinev.igor.ps.reader.PSReader;

public class ReaderWriterTest extends TestCase {
	
	private interface Action {
		void doAction(IDocument d);
	}
	static {
		DocumentFactory.setNamingMap(NamingMapTest.createNamingMapStub());
	}
	
	public static void doTest(String s) throws IOException, PSParseException {
		IReader r = new PSReader(new InputStreamReader(new ByteArrayInputStream(s.getBytes())));
		IDocument d = DocumentFactory.getDocumentFromReader(r);						
		StringWriter w = new StringWriter();
		d.write(w);
		assertEquals(s, w.toString());
	}
	
	public void test1() throws IOException, PSParseException {				
		doTest(TestMisc.inputStrings[0]);
	}	
	
	public void test2() throws IOException, PSParseException {				
		doTest(TestMisc.inputStrings[1]);
	}
	
	public void test3() throws IOException, PSParseException {				
		doTest(TestMisc.inputStrings[2]);
	}
	
	public void test4() throws IOException, PSParseException {				
		doTest(TestMisc.inputStrings[3]);
	}
	
	public void testReal1() throws PSParseException, IOException {
		File f = new File("org/sinev/igor/ps/tests/test1.ps");		
		FileReader fr = new FileReader(f);
		char[] in = new char[(int) f.length()];
		assertEquals(fr.read(in), f.length());
		fr.close();		
		IReader r = new PSReader(new FileReader(f));
		IDocument d = DocumentFactory.getDocumentFromReader(r);						
		StringWriter w = new StringWriter();
		d.write(w);
		String s = w.toString();
		for (int i = 0; i < in.length; i++)
			assertEquals(String.valueOf(i) + " ", in[i], s.charAt(i));
	}
	
	public void doTestAction(String fin, String fout, Action a) throws PSParseException, IOException {
		File f = new File(fin);			
		IReader r = new PSReader(new FileReader(f));
		IDocument d = DocumentFactory.getDocumentFromReader(r);								

		a.doAction(d);		
		StringWriter w = new StringWriter();
		d.write(w);
		String s = w.toString();
		FileReader fr = new FileReader(new File(fout));
		for (int i = 0; i < s.length(); i++) {
			int j = fr.read();
			assertFalse("Correct output already finished", j == -1);
			char c = (char) j;
			assertEquals(i + " ", s.charAt(i), c);
		}			
	}
	
	public void doTestPath(String fin, String fout) throws PSParseException, IOException {
		doTestAction(fin, fout, new Action() {
			public void doAction(IDocument d) {
				try {
					SizesDoubler.doubleSizes(d);
				} catch (PSParseException e) {
					e.printStackTrace();
				}				
			}
		});			
	}
	
	public void doTestSwapXY(String fin, String fout) throws PSParseException, IOException {
		doTestAction(fin, fout, new Action() {
			public void doAction(IDocument d) {
				try {
					XYSwapper.swapXY(d);
				} catch (PSParseException e) {
					e.printStackTrace();
				}				
			}			
		});			
	}
	
	public void testPath1() throws PSParseException, IOException  {
		doTestPath("org/sinev/igor/ps/tests/testsmall1.ps", "org/sinev/igor/ps/tests/testsmall1.out");
	}
	
	public void testPath2() throws PSParseException, IOException  {
		doTestPath("org/sinev/igor/ps/tests/testsmall2.ps", "org/sinev/igor/ps/tests/testsmall2.out");
	}
	
	public void testXY1() throws PSParseException, IOException  {
		doTestSwapXY("org/sinev/igor/ps/tests/testsmall1.ps", "org/sinev/igor/ps/tests/testsmall1swap.out");
	}
	
	public void testXY2() throws PSParseException, IOException  {
		doTestSwapXY("org/sinev/igor/ps/tests/testsmall2.ps", "org/sinev/igor/ps/tests/testsmall2swap.out");
	}
}
