package org.sinev.igor.psmisc;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.StringTokenizer;
import java.util.regex.Pattern;

import org.sinev.igor.ps.elementtype.ElementType;
import org.sinev.igor.ps.operatordescription.OperatorNamingMap;

/**
 * Simplifies generation of operator descriptions parsing 
 * HTML PS format description.
 * 
 * @author Igor Sinev
 */
public class ReferenceDescriptionParser {

	public static void addDescriptionByString(OperatorNamingMap m, String s, String group) {
		int i = s.indexOf("<9.5pt>");
		String description = removeTags(s.substring(i + 7)).trim();
		s = s.substring(0, i);
		i = s.indexOf("<b>");
		int j = s.indexOf(' ', i);
		m.addDescription(
				removeTags(s.substring(i + 3, j)).trim(), 
				parseSeveralTypes(s.substring(0, i)),
				parseSeveralTypes(s.substring(j)),
				group,
				description
			);		
	}
	
	private static String removeTags(String s) {
		StringBuffer r = new StringBuffer();
		while (true) {
			int i = s.indexOf('<');
			if (i == -1) { 
				r.append(s);
				break;
			}
			else {
				r.append(s.substring(0, i));
				s = s.substring(s.indexOf('>', i) + 1);
			}
		}
		return r.toString();
	}

	private static Pattern digitPattern = Pattern.compile("\\d*");	
	private static ElementType[] parseSeveralTypes(String s) {
		s = removeTags(s);
		StringTokenizer st = new StringTokenizer(s, "> ");
		ArrayList<ElementType> res = new ArrayList<ElementType>();		
		while (st.hasMoreTokens()) {
			String t = st.nextToken();
			if (t.length() == 1)
				continue;
			if (t.indexOf('<') == -1 && !digitPattern.matcher(t).matches()) {
				ElementType et = parseOneType(t);
				if (et != null)
					res.add(et);				
			}
		}
		return res.toArray(new ElementType[res.size()]);
	}
	
	private static HashMap<String, ElementType> typesMap = new HashMap<String, ElementType>();
	static {
		typesMap.put("int", ElementType.INT);
		typesMap.put("num", ElementType.NUMBER);
		typesMap.put("real", ElementType.REAL);
		typesMap.put("string", ElementType.INT);
	}
	/**
	 * Gets type of one element as it occurs in reference.
	 * No element ('-') is represented by null.
	 * 
	 * @param s string describing element.
	 * 
	 * @return obtained element.
	 */
	private static ElementType parseOneType(String s) {
		s = s.trim();		
		if (s.equals("-"))
			return null;
		ElementType t = typesMap.get(s);
		if (t == null)
			t = typesMap.get(s.substring(1, s.length()));
		return (t == null) ? ElementType.ANY_TYPE : t;
	}

}
